package FS;

import jade.core.Agent;
import jade.core.behaviours.*;
import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import jade.domain.AMSService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.*;
import jade.lang.acl.*;
import jade.core.AID;
import jade.domain.DFService;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Random;

public class Agent1  extends Agent {

	private static final long serialVersionUID = 1L;
	
	//Random
	Random ram = new Random();
	
	//Type of agent
	String type="Agent1";
	
	//Steps used in behavior
	int step=0;
	
	//Hunger and thirst levels
	int hunger = GConstants.hunger;
	int thirst = GConstants.thirst;
	
	//Numero de saludos recibidos
	int nHi=0;
	
	//Parameters used by the agent
	int n_ExistingAgents=-1;
	int n_TargetAgents=-1;
	
	//Reader of parameters
	String strparam;
	
	//Container and Controller
	AgentContainer a_Container;
	AgentController a_Controller;
	
	//Parameters of new Agent
	Object[] newAgParam = new Object[4];
	
	protected void SettingParameters()
	{
		strparam=Utilities.readArgument(GConstants.key_numberOfExistingAgents, getArguments());
		//System.out.println("param 1:"+strparam);
		if(Utilities.isStringAnInteger(strparam))
		{
			n_ExistingAgents=Integer.parseInt(strparam);
		}
		strparam=Utilities.readArgument(GConstants.key_numberOFAgents, getArguments());
		//System.out.println("param 2:"+strparam);
		if(Utilities.isStringAnInteger(strparam))
		{
			n_TargetAgents=Integer.parseInt(strparam);
		}
		
	}
	protected void CreateAnAgent(String type)
	{
		String name=type;
		if(name.contentEquals("Agent1"))
		{
			name="";
		}
		a_Container = getContainerController();
		try{
			newAgParam[0]=GConstants.key_numberOfExistingAgents;
			newAgParam[1]=n_ExistingAgents+1;
			newAgParam[2]=GConstants.key_numberOFAgents;
			newAgParam[3]=n_TargetAgents;
			//Only the first is important, the other just will be considered to make it
			newAgParam[0]=Utilities.parseArguments(newAgParam);
			int number = ram.nextInt(GConstants.namesList.size());
			a_Controller = a_Container.createNewAgent(name+" "+GConstants.namesList.get(number), "FS."+type, newAgParam);
			GConstants.namesList.remove(number);
			a_Controller.start();
		}
		catch(Exception e)
		{
			System.out.println("Error creating new agent: "+e.getMessage());
		}
	}
	
	
	public void sendMessageBroadCast(String message, int type)
	{
		AID myId = getAID();
		AMSAgentDescription [] agents = null;
		try {
			SearchConstraints constraints = new SearchConstraints();
			constraints.setMaxResults(new Long(-1));
			agents = AMSService.search( this, new AMSAgentDescription (), constraints );
		} catch (Exception e) {
			System.out.println("Error "+ e);
		}
		
		ACLMessage msg = new ACLMessage(type);
		msg.setContent(message);

		for (int i=0; i<agents.length;i++) {
			if(! myId.getName().contentEquals(agents[i].getName().getName()))
				msg.addReceiver( agents[i].getName() );	
		}
			
		System.out.println(getLocalName()+": "+message);
		send(msg);
		
	}
	
	public void sendMessage(String receiver, String message, int type)
	{
		ACLMessage msg = new ACLMessage(type);
		msg.setContent(message);
		msg.addReceiver( new AID(receiver,AID.ISLOCALNAME));
		System.out.println(getLocalName()+": "+message);
		send(msg);
	}
	
	public void replyMessage(ACLMessage msg, String message, int type)
	{
		ACLMessage reply=msg.createReply();
		reply.setContent(message);
		reply.setPerformative(type);
		System.out.println(getLocalName()+": "+message);
		send(reply);
			
	}
	
	protected void regService(String type, String service)
	{
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
	    sd.setType(type);      
	    sd.setName(service);
	    dfd.addServices(sd); 
	    try {
	        DFService.register(this, dfd); 
	      }
	      catch (FIPAException fe) {       
	        fe.printStackTrace();          
	      }
	}
	protected String seekService(String tipo, String nombre)
	{
		ServiceDescription ser = new ServiceDescription();
		DFAgentDescription des = new DFAgentDescription();
		DFAgentDescription[] serv_Availables;
		ser.setType(tipo);
		ser.setName(nombre);
		des.addServices(ser);
		try
		{
			serv_Availables = DFService.search(this, des);
			int i = ram.nextInt(serv_Availables.length);
			return serv_Availables[i].getName().getLocalName();
			
		}catch (Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	protected void setHungerAndThirst() {
		Random ram_number = new Random();
		hunger = ram_number.nextInt(30-15) + 15;
		thirst = ram_number.nextInt(40-30) + 30;
	}
	
	protected void generateReport() throws IOException {
		String content = new String(Files.readAllBytes(Paths.get(GConstants.listPath)));
		ArrayList<String> namesArray = new ArrayList<String>();
		ArrayList<Integer> foodArray = new ArrayList<Integer>();
		ArrayList<Integer> drinkArray = new ArrayList<Integer>();
		content = content.replace("|", "&&");
		System.out.println();
		System.out.println("=================== INFORME ===================");
		for (String order : content.split("&&")) {
			String [] order_parts = order.split(":");
			String name = order_parts[0];
			String product = order_parts[1];
			int amount = Integer.parseInt(order_parts[2]);
			if (!namesArray.contains(name)) {
				namesArray.add(name); foodArray.add(0); drinkArray.add(0);
			}
			int position = namesArray.indexOf(name);
			if (product.contains("comer"))	foodArray.set(position, foodArray.get(position) + amount);
			if (product.contains("beber"))	drinkArray.set(position, drinkArray.get(position) + amount);
		}
		for (int i = 0; i<namesArray.size(); i++) {
			System.out.println("     "+namesArray.get(i)+" ha comido "+foodArray.get(i)+ " unidades y ha bebido "+drinkArray.get(i)+" unidades."
					+ " Precio: " + (foodArray.get(i)*GConstants.foodUnitPrice+drinkArray.get(i)*GConstants.drinkUnitPrice) + "€");
		}
		System.out.println("===============================================");
		
		File file = new File(GConstants.listPath);
		file.delete();
	}
	
	protected void setup()
	{
		SettingParameters();
		setHungerAndThirst();
		addBehaviour(new MyBehaviour(this,1000));
		addBehaviour(new responseBehaviour(this));
	}
	
	class MyBehaviour extends TickerBehaviour {
		
		String waiter="";
		private static final long serialVersionUID = 1L;
		Agent a;

		public MyBehaviour (Agent a, int timemaps)
		{
			super(a, timemaps);
			this.a = a;
		}

		@Override
		public void onTick() {
			
			switch(step)
			{
			case 0:
				//This case will create a new Agent by each Agent existing until reach the target
				System.out.println("Hola yo soy: "+getLocalName());
				if (n_ExistingAgents==0)
				{
					regService(GConstants.t_service_host, GConstants.service_atender);
				}
				if(n_ExistingAgents < n_TargetAgents)
					{
						CreateAnAgent(type);
						GConstants.nPeopleParty++;
						//wait state
						step=-1;
					}else
					{
						step=1;
					}
				break;
			case 1:
				sendMessageBroadCast(GConstants.pattern_We_are_allI,ACLMessage.INFORM);
				step=2;
				break;
			case 2:
				sendMessageBroadCast(GConstants.pattern_SayHello+": soy "+getLocalName(),ACLMessage.INFORM);
				//step=3;
				break;
			case 3:
				int amount;
				if ((hunger <= 0) && (thirst <= 0) && (getLocalName().compareTo("Fred") != 0)) {
					String host;
					System.out.println(getLocalName() + " ya estoy bien servido/a, voy a por el abrigo y me despido");
					try {
						Thread.sleep(350);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					if ((host=seekService(GConstants.t_service_host,GConstants.service_atender))!= null)
					{
						sendMessage(host,GConstants.pattern_Saybye+":"+host,ACLMessage.INFORM);
					}
					GConstants.nPeopleParty--;
					a.doDelete();
				}
				else if ((waiter=seekService(GConstants.t_service_camarero,GConstants.service_atender))!= null) {
					amount = ram.nextInt(3)+1;
					if ((ram.nextInt(2)==1) && (thirst > 0)) {
						sendMessage(waiter,GConstants.pattern_order+":"+GConstants.type_drink+":"+amount,ACLMessage.INFORM);
						thirst -= amount;
					} else if (hunger > 0) {
						sendMessage(waiter,GConstants.pattern_order+":"+GConstants.type_food+":"+amount,ACLMessage.INFORM);	
						hunger -= amount;
					}
				}
				
				if (getLocalName().compareTo("Fred") == 0) {
					try {
						if (GConstants.nPeopleParty <= 0) {
//							Thread.sleep(2000);
							System.out.println();
							System.out.println(getLocalName()+": me he quedado solo, hora de acabar la fiesta");
							generateReport();
							a.doDelete();
						}
					} catch (IOException e) {
						System.out.println("No se ha podido completar el informe");
					}
				}
				break;
			default:
				break;
			}
		}
	}
	
	class responseBehaviour extends CyclicBehaviour{

		private static final long serialVersionUID = 1L;

		public responseBehaviour(Agent a) {
			super(a);
		}
		@Override
		public void action() {
			ACLMessage msg = receive();
			if (msg==null){return;}
			String [] param = msg.getContent().split(":");
			String senderName = msg.getSender().getLocalName();
			String resp="";
			switch(param[0])
			{
			case GConstants.pattern_We_are_allI:
				step=2;
				break;
			case GConstants.pattern_SayHello:
				resp = GConstants.pattern_RSayHello+":"+senderName;
				replyMessage(msg,resp,ACLMessage.INFORM);
				nHi++;
				if(nHi==n_TargetAgents-1) {
					step=3;
				}
				break;
			case GConstants.pattern_Saybye:
				resp = GConstants.pattern_Saybye+":"+senderName;
				replyMessage(msg,resp,ACLMessage.INFORM);
				break;
			}
		}
		
	}
	
}
